/*
  Copyright (c) 2004 Dirk Jagdmann <doj@cubic.org>

  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any
  purpose, including commercial applications, and to alter it and
  redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must
     not claim that you wrote the original software. If you use this
     software in a product, an acknowledgment in the product
     documentation would be appreciated but is not required.

  2. Altered source versions must be plainly marked as such, and must
     not be misrepresented as being the original software.

  3. This notice may not be removed or altered from any source
     distribution.
*/

// http://www.personal.uni-jena.de/~pfk/mpp/sv8/apetag.html

#include <string.h>
#include <stdint.h>
#include <iostream>
#include "doj/memorymap.hpp"

typedef char utf8_t;

enum {ReadOnly=1, IsFooter=0x20000000, ContainsFooter=0x40000000, ContainsHeader=0x80000000};

#pragma pack(push, 1)
struct APEheader
{
  char id[8];
  uint32_t version;
  uint32_t size;
  uint32_t items;
  uint32_t flags;
  uint64_t reserved;
};

struct APEitem
{
  uint32_t valueLen;
  uint32_t flags;
  char *key() { return reinterpret_cast<char*>(this)+sizeof(valueLen)+sizeof(flags); }
  utf8_t* value() { return key()+strlen(key())+1; }
  unsigned length() { return valueLen+strlen(key())+1+sizeof(valueLen)+sizeof(flags); }
};
#pragma pack(pop)

void printtags(char *filename)
{
  void *m=doj::memorymap(filename);
  if(!m)
    return;
  const int size=doj::memorymapsize(m);

  char *start=static_cast<char*>(m);
  const char *end=static_cast<char*>(m)+size;

  while(start!=end)
    {
      if(!strncmp(start, "APETAGEX", 8))
	break;
      start++;
    }

  if(start!=end)
    {
      APEheader *header=reinterpret_cast<APEheader*>(start);
      start+=sizeof(APEheader);
      for(unsigned i=0; i<header->items && start<end; ++i)
	{
	  APEitem *item=reinterpret_cast<APEitem*>(start);

	  std::cout << item->key() << ':';

	  utf8_t *value=item->value();
	  for(unsigned l=0; l<item->valueLen; l++, value++)
	    if(*value!=0xA && *value!=0xD)
	      std::cout << *value;

	  std::cout << std::endl;

	  start+=item->length();
	}
    }

  doj::memoryunmap(m);
}
